@@ -24,6 +24,7 @@
 #ifndef SUDO_COMPAT_H
 #define SUDO_COMPAT_H
 
+#include <sys/types.h>	/* for gid_t, mode_t, size_t, ssize_t, uid_t */
 #include <stdio.h>
 #if !defined(HAVE_VSNPRINTF) || !defined(HAVE_VASPRINTF) || \
     !defined(HAVE_VSYSLOG) || defined(PREFER_PORTABLE_SNPRINTF)
@@ -212,16 +213,21 @@
 #  define UTIME_NOW	-2L
 # endif
 #endif
-#if !defined(HAVE_OPENAT) || (!defined(HAVE_FUTIMENS) && !defined(HAVE_UTIMENSAT))
+#if !defined(HAVE_OPENAT) || (!defined(HAVE_FUTIMENS) && !defined(HAVE_UTIMENSAT)) || !defined(HAVE_FCHMODAT) || !defined(HAVE_FSTATAT) || !defined(HAVE_UNLINKAT)
 # ifndef AT_FDCWD
-#  define AT_FDCWD	-100
+#  define AT_FDCWD		-100
+# endif
+# ifndef AT_SYMLINK_NOFOLLOW
+#  define AT_SYMLINK_NOFOLLOW	0x02
 # endif
 #endif
 
-/* For pipe2() emulation. */
-#if !defined(HAVE_PIPE2) && defined(O_NONBLOCK) && (!defined(O_CLOEXEC) || O_CLOEXEC > 0xffffffff)
-# undef O_CLOEXEC
-# define O_CLOEXEC	0x80000000
+/* For dup3() and pipe2() emulation. */
+#if (!defined(HAVE_PIPE2) || !defined(HAVE_DUP3)) && defined(O_NONBLOCK)
+# if !defined(O_CLOEXEC) || O_CLOEXEC > 0xffffffff
+#  undef O_CLOEXEC
+#  define O_CLOEXEC	0x80000000
+# endif
 #endif
 
 /*
@@ -313,8 +319,8 @@ extern int errno;
 #endif
 
 /* Older systems may not support WCONTINUED */
-#ifndef WCONTINUED
-# define WCONTINUED	0
+#if !defined(WCONTINUED) && !defined(WIFCONTINUED)
+# define WCONTINUED		0
 # define WIFCONTINUED(x)	0
 #endif
 
@@ -385,14 +391,46 @@ int getdomainname(char *, size_t);
 # endif
 #endif /* __hpux && !__LP64__ */
 
+/*
+ * Older systems may lack fseeko(3), just use fseek(3) instead.
+ */
+#ifndef HAVE_FSEEKO
+# define fseeko(f, o, w)	fseek((f), (long)(o), (w))
+#endif
+
+/*
+ * Compatibility defines for OpenSSL 1.0.2 (not needed for 1.1.x)
+ */
+#if defined(HAVE_OPENSSL)
+# ifndef HAVE_X509_STORE_CTX_GET0_CERT
+#  define X509_STORE_CTX_get0_cert(x)   ((x)->cert)
+# endif
+# ifndef HAVE_ASN1_STRING_GET0_DATA
+#  define ASN1_STRING_get0_data(x)      ASN1_STRING_data(x)
+# endif
+# ifndef HAVE_TLS_CLIENT_METHOD
+#  define TLS_client_method()           SSLv23_client_method()
+# endif
+# ifndef HAVE_TLS_SERVER_METHOD
+#  define TLS_server_method()           SSLv23_server_method()
+# endif
+#endif /* HAVE_OPENSSL */
+
 /*
  * Functions "missing" from libc.
  * All libc replacements are prefixed with "sudo_" to avoid namespace issues.
  */
 
 struct passwd;
+struct stat;
 struct timespec;
+struct termios;
 
+#ifndef HAVE_CFMAKERAW
+__dso_public void sudo_cfmakeraw(struct termios *term);
+# undef cfmakeraw
+# define cfmakeraw(_a) sudo_cfmakeraw((_a))
+#endif /* HAVE_CFMAKERAW */
 #ifndef HAVE_CLOSEFROM
 __dso_public void sudo_closefrom(int);
 # undef closefrom
@@ -432,6 +470,16 @@ __dso_public int sudo_utimensat(int fd, const char *file, const struct timespec
 # undef utimensat
 # define utimensat(_a, _b, _c, _d) sudo_utimensat((_a), (_b), (_c), (_d))
 #endif /* HAVE_UTIMENSAT */
+#ifndef HAVE_FCHMODAT
+__dso_public int sudo_fchmodat(int dfd, const char *path, mode_t mode, int flag);
+# undef fchmodat
+# define fchmodat(_a, _b, _c, _d) sudo_fchmodat((_a), (_b), (_c), (_d))
+#endif /* HAVE_FCHMODAT */
+#ifndef HAVE_FSTATAT
+__dso_public int sudo_fstatat(int dfd, const char *path, struct stat *sb, int flag);
+# undef fstatat
+# define fstatat(_a, _b, _c, _d) sudo_fstatat((_a), (_b), (_c), (_d))
+#endif /* HAVE_FSTATAT */
 #ifndef HAVE_FUTIMENS
 __dso_public int sudo_futimens(int fd, const struct timespec *times);
 # undef futimens
@@ -525,7 +573,7 @@ __dso_public int sudo_str2sig(const char *signame, int *signum);
 # undef str2sig
 # define str2sig(_a, _b) sudo_str2sig((_a), (_b))
 #endif /* HAVE_STR2SIG */
-#if !defined(HAVE_INET_NTOP) && defined(SUDO_NET_IFS_C)
+#if !defined(HAVE_INET_NTOP) && defined(NEED_INET_NTOP)
 __dso_public char *sudo_inet_ntop(int af, const void *src, char *dst, socklen_t size);
 # undef inet_ntop
 # define inet_ntop(_a, _b, _c, _d) sudo_inet_ntop((_a), (_b), (_c), (_d))
@@ -540,11 +588,6 @@ __dso_public const char *sudo_getprogname(void);
 # undef getprogname
 # define getprogname() sudo_getprogname()
 #endif /* HAVE_GETPROGNAME */
-#ifndef HAVE_SETPROGNAME
-__dso_public void sudo_setprogname(const char *name);
-# undef setprogname
-# define setprogname(_a) sudo_setprogname(_a)
-#endif /* HAVE_SETPROGNAME */
 #ifndef HAVE_REALLOCARRAY
 __dso_public void *sudo_reallocarray(void *ptr, size_t nmemb, size_t size);
 # undef reallocarray
@@ -555,6 +598,11 @@ __dso_public void sudo_vsyslog(int pri, const char *fmt, va_list ap);
 # undef vsyslog
 # define vsyslog(_a, _b, _c) sudo_vsyslog((_a), (_b), (_c))
 #endif /* HAVE_VSYSLOG */
+#ifndef HAVE_DUP3
+__dso_public int sudo_dup3(int oldd, int newd, int flags);
+# undef dup3
+# define dup3(_a, _b, _c) sudo_dup3((_a), (_b), (_c))
+#endif /* HAVE_DUP3 */
 #ifndef HAVE_PIPE2
 __dso_public int sudo_pipe2(int fildes[2], int flags);
 # undef pipe2
