@@ -19,6 +19,8 @@
 #ifndef SUDO_IOLOG_H
 #define SUDO_IOLOG_H
 
+#include <sys/types.h>	/* for gid_t, mode_t, size_t, ssize_t, uid_t */
+
 #ifdef HAVE_ZLIB_H
 # include <zlib.h>	/* for gzFile */
 #endif
@@ -139,6 +141,7 @@ int iolog_openat(int fdf, const char *path, int flags);
 off_t iolog_seek(struct iolog_file *iol, off_t offset, int whence);
 ssize_t iolog_read(struct iolog_file *iol, void *buf, size_t nbytes, const char **errstr);
 ssize_t iolog_write(struct iolog_file *iol, const void *buf, size_t len, const char **errstr);
+void iolog_clearerr(struct iolog_file *iol);
 void iolog_rewind(struct iolog_file *iol);
 void iolog_set_compress(bool);
 void iolog_set_defaults(void);
