@@ -23,7 +23,6 @@
 
 #include <config.h>
 
-#include <sys/types.h>
 #include <sys/resource.h>
 
 #include <stdio.h>
@@ -32,12 +31,10 @@
 #include <usersec.h>
 #include <uinfo.h>
 
-#define DEFAULT_TEXT_DOMAIN	"sudo"
-#include "sudo_gettext.h"	/* must be included before sudo_compat.h */
-
 #include "sudo_compat.h"
-#include "sudo_fatal.h"
 #include "sudo_debug.h"
+#include "sudo_fatal.h"
+#include "sudo_gettext.h"
 #include "sudo_util.h"
 
 #ifdef HAVE_GETUSERATTR
