@@ -37,14 +37,8 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 
-#include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
+#include <string.h>
 #include <limits.h>
 #include <netdb.h>
 #include <errno.h>
