@@ -23,38 +23,19 @@
 
 #include <config.h>
 
-#include <sys/types.h>
-#include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
+#include <string.h>
 
 #include "sudo_compat.h"
 #include "sudo_util.h"
 
 #ifdef HAVE_GETPROGNAME
 
-# ifndef HAVE_SETPROGNAME
-/* Assume __progname if have getprogname(3) but not setprogname(3). */
-extern const char *__progname;
-
 void
-sudo_setprogname(const char *name)
-{
-    const char *slash = strrchr(name, '/');
-    __progname = slash ? slash + 1 : name;
-}
-# endif
-
-void
-initprogname2(const char *name, const char * const * allowed)
+initprogname(const char *name)
 {
+# ifdef HAVE_SETPROGNAME
     const char *progname;
-    int i;
 
     /* Fall back on "name" if getprogname() returns an empty string. */
     if ((progname = getprogname()) != NULL && *progname != '\0')
@@ -64,21 +45,10 @@ initprogname2(const char *name, const char * const * allowed)
     if (name[0] == 'l' && name[1] == 't' && name[2] == '-' && name[3] != '\0')
 	name += 3;
 
-    /* Check allow list if present (first element is the default). */
-    if (allowed != NULL) {
-	for (i = 0; ; i++) {
-	    if (allowed[i] == NULL) {
-		name = allowed[0];
-		break;
-	    }
-	    if (strcmp(allowed[i], name) == 0)
-		break;
-	}
-    }
-
     /* Update internal progname if needed. */
     if (name != progname)
 	setprogname(name);
+# endif
     return;
 }
 
@@ -87,9 +57,8 @@ initprogname2(const char *name, const char * const * allowed)
 static const char *progname = "";
 
 void
-initprogname2(const char *name, const char * const * allowed)
+initprogname(const char *name)
 {
-    int i;
 # ifdef HAVE___PROGNAME
     extern const char *__progname;
 
@@ -107,18 +76,6 @@ initprogname2(const char *name, const char * const * allowed)
     if (progname[0] == 'l' && progname[1] == 't' && progname[2] == '-' &&
 	progname[3] != '\0')
 	progname += 3;
-
-    /* Check allow list if present (first element is the default). */
-    if (allowed != NULL) {
-	for (i = 0; ; i++) {
-	    if (allowed[i] == NULL) {
-		progname = allowed[0];
-		break;
-	    }
-	    if (strcmp(allowed[i], progname) == 0)
-		break;
-	}
-    }
 }
 
 const char *
@@ -126,17 +83,4 @@ sudo_getprogname(void)
 {
     return progname;
 }
-
-void
-sudo_setprogname(const char *name)
-{
-    const char *slash = strrchr(name, '/');
-    progname = slash ? slash + 1 : name;
-}
 #endif /* !HAVE_GETPROGNAME */
-
-void
-initprogname(const char *name)
-{
-    initprogname2(name, NULL);
-}
