@@ -1,5 +1,7 @@
 /*	$OpenBSD: vfprintf.c,v 1.67 2014/12/21 00:23:30 daniel Exp $	*/
-/*-
+/* 
+ * SPDX-License-Identifier: BSD-3-Clause
+ *
  * Copyright (c) 1999-2005, 2008, 2010-2016
  *      Todd C. Miller <Todd.Miller@sudo.ws>
  * Copyright (c) 1990, 1993
@@ -45,7 +47,6 @@
     !defined(HAVE_VASPRINTF) || !defined(HAVE_ASPRINTF) || \
     defined(PREFER_PORTABLE_SNPRINTF)
 
-#include <sys/types.h>
 #include <sys/mman.h>
 
 #include <errno.h>
@@ -62,12 +63,7 @@
 # include <inttypes.h>
 #endif
 #include <stdio.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
+#include <string.h>
 #include <unistd.h>
 #ifdef PRINTF_WIDE_CHAR
 # include <wchar.h>
