@@ -25,14 +25,8 @@
 
 #ifndef HAVE_STR2SIG
 
-#include <sys/types.h>
-
 #include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
+#include <string.h>
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif /* HAVE_STRINGS_H */
