@@ -23,16 +23,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
+#include <string.h>
 #include <ctype.h>
-#include <errno.h>
-#include <grp.h>
-#include <pwd.h>
 
 #include <netinet/in.h>
 #include <arpa/inet.h>
