@@ -1,11 +1,21 @@
 #!/bin/sh
 #
-# Test #include facility
+# Test @include facility
 #
 
 MYUID=`\ls -ln $TESTDIR/test2.inc | awk '{print $3}'`
 MYGID=`\ls -ln $TESTDIR/test2.inc | awk '{print $4}'`
 exec 2>&1
+
+echo "Testing @include"
+echo ""
+./testsudoers -U $MYUID -G $MYGID root id <<EOF
+@include $TESTDIR/test2.inc
+EOF
+
+echo ""
+echo "Testing #include"
+echo ""
 ./testsudoers -U $MYUID -G $MYGID root id <<EOF
 #include $TESTDIR/test2.inc
 EOF
