@@ -28,15 +28,8 @@
 
 #include <config.h>
 
-#include <sys/types.h>
 #include <stdio.h>
-#include <stdlib.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
+#include <string.h>
 
 #include <netinet/in.h>
 #include <arpa/inet.h>
