@@ -23,9 +23,7 @@
 
 #include <config.h>
 
-#include <sys/types.h>
 #include <stdio.h>
-#include <stdlib.h>
 #include <stdarg.h>
 #include <errno.h>
 
