@@ -23,22 +23,14 @@
 
 #include <config.h>
 
-#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
+#include <string.h>
 #include <unistd.h>
 #ifdef HAVE_PRIV_SET
 # include <priv.h>
 #endif
 #include <errno.h>
-#include <fcntl.h>
-#include <signal.h>
 
 #include "sudo.h"
 #include "sudo_exec.h"
