@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 2009-2021 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 2009-2020 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -30,12 +30,7 @@
 #include <sys/ioctl.h>
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
+#include <string.h>
 #include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -43,7 +38,6 @@
 #include <termios.h>		/* for struct winsize on HP-UX */
 
 #include "sudo.h"
-#include "sudo_event.h"
 #include "sudo_exec.h"
 #include "sudo_plugin.h"
 #include "sudo_plugin_int.h"
@@ -138,7 +132,7 @@ pty_cleanup(void)
 
 /*
  * Allocate a pty if /dev/tty is a tty.
- * Fills in io_fds[SFD_USERTTY], io_fds[SFD_MASTER], io_fds[SFD_SLAVE]
+ * Fills in io_fds[SFD_USERTTY], io_fds[SFD_LEADER], io_fds[SFD_FOLLOWER]
  * and ptyname globals.
  */
 static bool
@@ -153,9 +147,9 @@ pty_setup(struct command_details *details, const char *tty)
 	debug_return_bool(false);
     }
 
-    if (!get_pty(&io_fds[SFD_MASTER], &io_fds[SFD_SLAVE],
-	ptyname, sizeof(ptyname), details->cred.euid))
-	sudo_fatal("%s", U_("unable to allocate pty"));
+    if (!get_pty(&io_fds[SFD_LEADER], &io_fds[SFD_FOLLOWER],
+	ptyname, sizeof(ptyname), details->euid))
+	sudo_fatal(U_("unable to allocate pty"));
 
     /* Update tty name in command details (used by SELinux and AIX). */
     details->tty = ptyname;
@@ -164,30 +158,30 @@ pty_setup(struct command_details *details, const char *tty)
     if (ISSET(details->flags, CD_SET_UTMP)) {
 	utmp_user =
 	    details->utmp_user ? details->utmp_user : user_details.username;
-	utmp_login(tty, ptyname, io_fds[SFD_SLAVE], utmp_user);
+	utmp_login(tty, ptyname, io_fds[SFD_FOLLOWER], utmp_user);
     }
 
     sudo_debug_printf(SUDO_DEBUG_INFO,
-	"%s: %s fd %d, pty master fd %d, pty slave fd %d",
-	__func__, _PATH_TTY, io_fds[SFD_USERTTY], io_fds[SFD_MASTER],
-	io_fds[SFD_SLAVE]);
+	"%s: %s fd %d, pty leader fd %d, pty follower fd %d",
+	__func__, _PATH_TTY, io_fds[SFD_USERTTY], io_fds[SFD_LEADER],
+	io_fds[SFD_FOLLOWER]);
 
     debug_return_bool(true);
 }
 
 /*
- * Make the tty slave the controlling tty.
+ * Make the tty follower the controlling tty.
  * This is only used by the monitor but ptyname[] is static.
  */
 int
 pty_make_controlling(void)
 {
-    if (io_fds[SFD_SLAVE] != -1) {
+    if (io_fds[SFD_FOLLOWER] != -1) {
 #ifdef TIOCSCTTY
-	if (ioctl(io_fds[SFD_SLAVE], TIOCSCTTY, NULL) != 0)
+	if (ioctl(io_fds[SFD_FOLLOWER], TIOCSCTTY, NULL) != 0)
 	    return -1;
 #else
-	/* Set controlling tty by reopening pty slave. */
+	/* Set controlling tty by reopening pty follower. */
 	int fd = open(ptyname, O_RDWR);
 	if (fd == -1)
 	    return -1;
@@ -202,6 +196,7 @@ static bool
 log_ttyin(const char *buf, unsigned int n, struct io_buffer *iob)
 {
     struct plugin_container *plugin;
+    const char *errstr = NULL;
     sigset_t omask;
     bool ret = true;
     debug_decl(log_ttyin, SUDO_DEBUG_EXEC);
@@ -212,11 +207,17 @@ log_ttyin(const char *buf, unsigned int n, struct io_buffer *iob)
 	    int rc;
 
 	    sudo_debug_set_active_instance(plugin->debug_instance);
-	    rc = plugin->u.io->log_ttyin(buf, n);
+	    rc = plugin->u.io->log_ttyin(buf, n, &errstr);
 	    if (rc <= 0) {
 		if (rc < 0) {
 		    /* Error: disable plugin's I/O function. */
 		    plugin->u.io->log_ttyin = NULL;
+		    audit_error(plugin->name, SUDO_IO_PLUGIN,
+			errstr ? errstr : _("I/O plugin error"), NULL);
+		} else {
+		    audit_reject(plugin->name, SUDO_IO_PLUGIN,
+			errstr ? errstr : _("command rejected by I/O plugin"),
+			NULL);
 		}
 		ret = false;
 		break;
@@ -234,6 +235,7 @@ static bool
 log_stdin(const char *buf, unsigned int n, struct io_buffer *iob)
 {
     struct plugin_container *plugin;
+    const char *errstr = NULL;
     sigset_t omask;
     bool ret = true;
     debug_decl(log_stdin, SUDO_DEBUG_EXEC);
@@ -244,11 +246,17 @@ log_stdin(const char *buf, unsigned int n, struct io_buffer *iob)
 	    int rc;
 
 	    sudo_debug_set_active_instance(plugin->debug_instance);
-	    rc = plugin->u.io->log_stdin(buf, n);
+	    rc = plugin->u.io->log_stdin(buf, n, &errstr);
 	    if (rc <= 0) {
 		if (rc < 0) {
 		    /* Error: disable plugin's I/O function. */
 		    plugin->u.io->log_stdin = NULL;
+		    audit_error(plugin->name, SUDO_IO_PLUGIN,
+			errstr ? errstr : _("I/O plugin error"), NULL);
+		} else {
+		    audit_reject(plugin->name, SUDO_IO_PLUGIN,
+			errstr ? errstr : _("command rejected by I/O plugin"),
+			NULL);
 		}
 		ret = false;
 		break;
@@ -266,6 +274,7 @@ static bool
 log_ttyout(const char *buf, unsigned int n, struct io_buffer *iob)
 {
     struct plugin_container *plugin;
+    const char *errstr = NULL;
     sigset_t omask;
     bool ret = true;
     debug_decl(log_ttyout, SUDO_DEBUG_EXEC);
@@ -276,11 +285,17 @@ log_ttyout(const char *buf, unsigned int n, struct io_buffer *iob)
 	    int rc;
 
 	    sudo_debug_set_active_instance(plugin->debug_instance);
-	    rc = plugin->u.io->log_ttyout(buf, n);
+	    rc = plugin->u.io->log_ttyout(buf, n, &errstr);
 	    if (rc <= 0) {
 		if (rc < 0) {
 		    /* Error: disable plugin's I/O function. */
 		    plugin->u.io->log_ttyout = NULL;
+		    audit_error(plugin->name, SUDO_IO_PLUGIN,
+			errstr ? errstr : _("I/O plugin error"), NULL);
+		} else {
+		    audit_reject(plugin->name, SUDO_IO_PLUGIN,
+			errstr ? errstr : _("command rejected by I/O plugin"),
+			NULL);
 		}
 		ret = false;
 		break;
@@ -309,6 +324,7 @@ static bool
 log_stdout(const char *buf, unsigned int n, struct io_buffer *iob)
 {
     struct plugin_container *plugin;
+    const char *errstr = NULL;
     sigset_t omask;
     bool ret = true;
     debug_decl(log_stdout, SUDO_DEBUG_EXEC);
@@ -319,11 +335,17 @@ log_stdout(const char *buf, unsigned int n, struct io_buffer *iob)
 	    int rc;
 
 	    sudo_debug_set_active_instance(plugin->debug_instance);
-	    rc = plugin->u.io->log_stdout(buf, n);
+	    rc = plugin->u.io->log_stdout(buf, n, &errstr);
 	    if (rc <= 0) {
 		if (rc < 0) {
 		    /* Error: disable plugin's I/O function. */
 		    plugin->u.io->log_stdout = NULL;
+		    audit_error(plugin->name, SUDO_IO_PLUGIN,
+			errstr ? errstr : _("I/O plugin error"), NULL);
+		} else {
+		    audit_reject(plugin->name, SUDO_IO_PLUGIN,
+			errstr ? errstr : _("command rejected by I/O plugin"),
+			NULL);
 		}
 		ret = false;
 		break;
@@ -352,6 +374,7 @@ static bool
 log_stderr(const char *buf, unsigned int n, struct io_buffer *iob)
 {
     struct plugin_container *plugin;
+    const char *errstr = NULL;
     sigset_t omask;
     bool ret = true;
     debug_decl(log_stderr, SUDO_DEBUG_EXEC);
@@ -362,11 +385,17 @@ log_stderr(const char *buf, unsigned int n, struct io_buffer *iob)
 	    int rc;
 
 	    sudo_debug_set_active_instance(plugin->debug_instance);
-	    rc = plugin->u.io->log_stderr(buf, n);
+	    rc = plugin->u.io->log_stderr(buf, n, &errstr);
 	    if (rc <= 0) {
 		if (rc < 0) {
 		    /* Error: disable plugin's I/O function. */
 		    plugin->u.io->log_stderr = NULL;
+		    audit_error(plugin->name, SUDO_IO_PLUGIN,
+			errstr ? errstr : _("I/O plugin error"), NULL);
+		} else {
+		    audit_reject(plugin->name, SUDO_IO_PLUGIN,
+			errstr ? errstr : _("command rejected by I/O plugin"),
+			NULL);
 		}
 		ret = false;
 		break;
@@ -395,6 +424,7 @@ static void
 log_suspend(int signo)
 {
     struct plugin_container *plugin;
+    const char *errstr = NULL;
     sigset_t omask;
     debug_decl(log_suspend, SUDO_DEBUG_EXEC);
 
@@ -406,12 +436,12 @@ log_suspend(int signo)
 	    int rc;
 
 	    sudo_debug_set_active_instance(plugin->debug_instance);
-	    rc = plugin->u.io->log_suspend(signo);
+	    rc = plugin->u.io->log_suspend(signo, &errstr);
 	    if (rc <= 0) {
-		if (rc < 0) {
-		    /* Error: disable plugin's I/O function. */
-		    plugin->u.io->log_suspend = NULL;
-		}
+		/* Error: disable plugin's I/O function. */
+		plugin->u.io->log_suspend = NULL;
+		audit_error(plugin->name, SUDO_IO_PLUGIN,
+		    errstr ? errstr : _("error logging suspend"), NULL);
 		break;
 	    }
 	}
@@ -427,6 +457,7 @@ static void
 log_winchange(unsigned int rows, unsigned int cols)
 {
     struct plugin_container *plugin;
+    const char *errstr = NULL;
     sigset_t omask;
     debug_decl(log_winchange, SUDO_DEBUG_EXEC);
 
@@ -438,12 +469,12 @@ log_winchange(unsigned int rows, unsigned int cols)
 	    int rc;
 
 	    sudo_debug_set_active_instance(plugin->debug_instance);
-	    rc = plugin->u.io->change_winsize(rows, cols);
+	    rc = plugin->u.io->change_winsize(rows, cols, &errstr);
 	    if (rc <= 0) {
-		if (rc < 0) {
-		    /* Error: disable plugin's I/O function. */
-		    plugin->u.io->change_winsize = NULL;
-		}
+		/* Error: disable plugin's I/O function. */
+		plugin->u.io->change_winsize = NULL;
+		audit_error(plugin->name, SUDO_IO_PLUGIN,
+		    errstr ? errstr : _("error changing window size"), NULL);
 		break;
 	    }
 	}
@@ -624,7 +655,7 @@ read_callback(int fd, int what, void *v)
     switch (n) {
 	case -1:
 	    if (got_sigttin) {
-		/* Schedule SIGTTIN to be forwared to the command. */
+		/* Schedule SIGTTIN to be forwarded to the command. */
 		schedule_signal(iob->ec, SIGTTIN);
 	    }
 	    if (errno == EAGAIN || errno == EINTR)
@@ -729,7 +760,7 @@ write_callback(int fd, int what, void *v)
 	    break;
 	case EINTR:
 	    if (got_sigttou) {
-		/* Schedule SIGTTOU to be forwared to the command. */
+		/* Schedule SIGTTOU to be forwarded to the command. */
 		schedule_signal(iob->ec, SIGTTOU);
 	    }
 	    /* FALLTHROUGH */
@@ -809,7 +840,7 @@ io_buf_new(int rfd, int wfd,
 }
 
 /*
- * We already closed the slave pty so reads from the master will not block.
+ * We already closed the follower so reads from the leader will not block.
  */
 static void
 pty_finish(struct command_status *cstat)
@@ -856,7 +887,7 @@ static void
 send_command_status(struct exec_closure_pty *ec, int type, int val)
 {
     struct monitor_message *msg;
-    debug_decl(send_command, SUDO_DEBUG_EXEC)
+    debug_decl(send_command, SUDO_DEBUG_EXEC);
 
     if ((msg = calloc(1, sizeof(*msg))) == NULL)
 	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
@@ -880,7 +911,7 @@ static void
 schedule_signal(struct exec_closure_pty *ec, int signo)
 {
     char signame[SIG2STR_MAX];
-    debug_decl(schedule_signal, SUDO_DEBUG_EXEC)
+    debug_decl(schedule_signal, SUDO_DEBUG_EXEC);
 
     if (signo == 0)
 	debug_return;
@@ -904,7 +935,7 @@ backchannel_cb(int fd, int what, void *v)
     struct exec_closure_pty *ec = v;
     struct command_status cstat;
     ssize_t nread;
-    debug_decl(backchannel_cb, SUDO_DEBUG_EXEC)
+    debug_decl(backchannel_cb, SUDO_DEBUG_EXEC);
 
     /*
      * Read command status from the monitor.
@@ -999,7 +1030,7 @@ handle_sigchld_pty(struct exec_closure_pty *ec)
 {
     int n, status;
     pid_t pid;
-    debug_decl(handle_sigchld_pty, SUDO_DEBUG_EXEC)
+    debug_decl(handle_sigchld_pty, SUDO_DEBUG_EXEC);
 
     /*
      * Monitor process was signaled; wait for it as needed.
@@ -1051,7 +1082,7 @@ signal_cb_pty(int signo, int what, void *v)
     struct sudo_ev_siginfo_container *sc = v;
     struct exec_closure_pty *ec = sc->closure;
     char signame[SIG2STR_MAX];
-    debug_decl(signal_cb_pty, SUDO_DEBUG_EXEC)
+    debug_decl(signal_cb_pty, SUDO_DEBUG_EXEC);
 
     if (ec->monitor_pid == -1)
 	debug_return;
@@ -1085,7 +1116,7 @@ signal_cb_pty(int signo, int what, void *v)
 		debug_return;
 	    }
 	}
-	/* Schedule signal to be forwared to the command. */
+	/* Schedule signal to be forwarded to the command. */
 	schedule_signal(ec, signo);
 	break;
     }
@@ -1104,7 +1135,7 @@ fwdchannel_cb(int sock, int what, void *v)
     char signame[SIG2STR_MAX];
     struct monitor_message *msg;
     ssize_t nsent;
-    debug_decl(fwdchannel_cb, SUDO_DEBUG_EXEC)
+    debug_decl(fwdchannel_cb, SUDO_DEBUG_EXEC);
 
     while ((msg = TAILQ_FIRST(&ec->monitor_messages)) != NULL) {
 	switch (msg->cstat.type) {
@@ -1161,7 +1192,7 @@ static void
 fill_exec_closure_pty(struct exec_closure_pty *ec, struct command_status *cstat,
     struct command_details *details, pid_t ppgrp, int backchannel)
 {
-    debug_decl(fill_exec_closure_pty, SUDO_DEBUG_EXEC)
+    debug_decl(fill_exec_closure_pty, SUDO_DEBUG_EXEC);
 
     /* Fill in the non-event part of the closure. */
     ec->cmnd_pid = -1;
@@ -1177,9 +1208,8 @@ fill_exec_closure_pty(struct exec_closure_pty *ec, struct command_status *cstat,
     cstat->val = 0;
 
     /* Setup event base and events. */
-    ec->evbase = sudo_ev_base_alloc();
-    if (ec->evbase == NULL)
-	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+    ec->evbase = details->evbase;
+    details->evbase = NULL;
 
     /* Event for command status via backchannel. */
     ec->backchannel_event = sudo_ev_alloc(backchannel,
@@ -1280,7 +1310,7 @@ static void
 free_exec_closure_pty(struct exec_closure_pty *ec)
 {
     struct monitor_message *msg;
-    debug_decl(free_exec_closure_pty, SUDO_DEBUG_EXEC)
+    debug_decl(free_exec_closure_pty, SUDO_DEBUG_EXEC);
 
     sudo_ev_base_free(ec->evbase);
     sudo_ev_free(ec->backchannel_event);
@@ -1305,7 +1335,7 @@ free_exec_closure_pty(struct exec_closure_pty *ec)
 }
 
 /*
- * Execute a command in a pty, potentially with I/O loggging, and
+ * Execute a command in a pty, potentially with I/O logging, and
  * wait for it to finish.
  * This is a little bit tricky due to how POSIX job control works and
  * we fact that we have two different controlling terminals to deal with.
@@ -1323,7 +1353,7 @@ exec_pty(struct command_details *details, struct command_status *cstat)
     struct stat sb;
     pid_t ppgrp;
     int sv[2];
-    debug_decl(exec_pty, SUDO_DEBUG_EXEC)
+    debug_decl(exec_pty, SUDO_DEBUG_EXEC);
 
     /*
      * Allocate a pty.
@@ -1394,19 +1424,19 @@ exec_pty(struct command_details *details, struct command_status *cstat)
      * In background mode there is no stdin.
      */
     if (!ISSET(details->flags, CD_BACKGROUND))
-	io_fds[SFD_STDIN] = io_fds[SFD_SLAVE];
-    io_fds[SFD_STDOUT] = io_fds[SFD_SLAVE];
-    io_fds[SFD_STDERR] = io_fds[SFD_SLAVE];
+	io_fds[SFD_STDIN] = io_fds[SFD_FOLLOWER];
+    io_fds[SFD_STDOUT] = io_fds[SFD_FOLLOWER];
+    io_fds[SFD_STDERR] = io_fds[SFD_FOLLOWER];
 
     if (io_fds[SFD_USERTTY] != -1) {
-	/* Read from /dev/tty, write to pty master */
+	/* Read from /dev/tty, write to pty leader */
 	if (!ISSET(details->flags, CD_BACKGROUND)) {
-	    io_buf_new(io_fds[SFD_USERTTY], io_fds[SFD_MASTER],
+	    io_buf_new(io_fds[SFD_USERTTY], io_fds[SFD_LEADER],
 		log_ttyin, &ec, &iobufs);
 	}
 
-	/* Read from pty master, write to /dev/tty */
-	io_buf_new(io_fds[SFD_MASTER], io_fds[SFD_USERTTY],
+	/* Read from pty leader, write to /dev/tty */
+	io_buf_new(io_fds[SFD_LEADER], io_fds[SFD_USERTTY],
 	    log_ttyout, &ec, &iobufs);
 
 	/* Are we the foreground process? */
@@ -1481,8 +1511,8 @@ exec_pty(struct command_details *details, struct command_status *cstat)
     }
 
     if (foreground) {
-	/* Copy terminal attrs from user tty -> pty slave. */
-	if (!sudo_term_copy(io_fds[SFD_USERTTY], io_fds[SFD_SLAVE])) {
+	/* Copy terminal attrs from user tty -> pty follower. */
+	if (!sudo_term_copy(io_fds[SFD_USERTTY], io_fds[SFD_FOLLOWER])) {
             sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_ERRNO,
                 "%s: unable to copy terminal settings to pty", __func__);
 	    foreground = false;
@@ -1536,20 +1566,20 @@ exec_pty(struct command_details *details, struct command_status *cstat)
             sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_ERRNO,
                 "%s: unable to send status to parent", __func__);
 	}
-	_exit(1);
+	_exit(EXIT_FAILURE);
     }
 
     /*
-     * We close the pty slave so only the monitor and command have a
+     * We close the pty follower so only the monitor and command have a
      * reference to it.  This ensures that we can don't block reading
-     * from the master when the command and monitor have exited.
+     * from the leader when the command and monitor have exited.
      */
-    if (io_fds[SFD_SLAVE] != -1) {
-	close(io_fds[SFD_SLAVE]);
-	io_fds[SFD_SLAVE] = -1;
+    if (io_fds[SFD_FOLLOWER] != -1) {
+	close(io_fds[SFD_FOLLOWER]);
+	io_fds[SFD_FOLLOWER] = -1;
     }
 
-    /* Tell the monitor to continue now that the slave is closed. */
+    /* Tell the monitor to continue now that the follower is closed. */
     cstat->type = CMD_SIGNO;
     cstat->val = 0;
     while (send(sv[0], cstat, sizeof(*cstat), 0) == -1) {
@@ -1592,8 +1622,8 @@ exec_pty(struct command_details *details, struct command_status *cstat)
     setlocale(LC_ALL, "C");
 
     /*
-     * In the event loop we pass input from user tty to master
-     * and pass output from master to stdout and IO plugin.
+     * In the event loop we pass input from user tty to leader
+     * and pass output from leader to stdout and IO plugin.
      * Try to recover on ENXIO, it means the tty was revoked.
      */
     add_io_events(ec.evbase);
