@@ -27,20 +27,10 @@
 
 #include <config.h>
 
-#include <sys/types.h>
-
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
-#include <unistd.h>
+#include <string.h>
 #include <ctype.h>
-#include <grp.h>
-#include <pwd.h>
 #include <assert.h>
 
 #include <sudo_usage.h>
