@@ -23,23 +23,12 @@
 
 #include <config.h>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <stdio.h>
-#include <stdlib.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
-#include <unistd.h>
 #ifdef HAVE_PROJECT_H
 # include <project.h>
 # include <sys/task.h>
+# include <errno.h>
+# include <pwd.h>
 #endif
-#include <errno.h>
-#include <pwd.h>
 
 #include "sudo.h"
 #include "sudo_dso.h"
