@@ -23,17 +23,9 @@
 
 #include <config.h>
 
-#include <sys/types.h>
 #include <sys/time.h>
 #include <sys/wait.h>
-#include <stdio.h>
-#include <stdlib.h>
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
+#include <string.h>
 #include <unistd.h>
 #include <time.h>
 #if defined(HAVE_UTMPS_H)
